@@ -4,27 +4,32 @@
 # @title "Gets the default device options"
 #
 # \description{
-#  @get "title" as given by predefined devices options adjusted for 
+#  @get "title" as given by predefined devices options adjusted for
 #  the default arguments of the device function.
 # }
-# 
+#
 # @synopsis
 #
 # \arguments{
-#   \item{type}{A @character string or a device @function specifying 
-#      the device to be queried.}
+#   \item{type}{A @character string or a device @function specifying
+#      the device to be queried.  May also be a @vector, for querying
+#      device options for multiple devices.}
 #   \item{custom}{If @TRUE, also the default settings specific to this
 #      function is returned. For more details, see below.}
 #   \item{special}{A @logical.  For more details, see below.}
+#   \item{drop}{If @TRUE and only one device type is queried, then
+#      a @list is returned, otherwise a @matrix.}
+#   \item{options}{Optional named @list of settings.}
 #   \item{...}{Optional named arguments for setting new defaults.
 #      For more details, see below.}
 #   \item{reset}{If @TRUE, the device options are reset to R defaults.}
 # }
 #
 # \value{
-#   Returns a named @list.
-#   If the requested device does not exists (certain devices are OS
-#   specific), then an empty @list is returned.
+#   If \code{drop=TRUE} and a single device is queries, a named @list is
+#   returned, otherwise a @matrix is returned.
+#   If a requested device is not implemented/supported on the current system,
+#   then "empty" results are returned.
 #   If options were set, that is, if named options were specified via
 #   \code{...}, then the list is returned invisibly, otherwise not.
 # }
@@ -42,7 +47,7 @@
 #  any device function needs to query also such options, which for instance
 #  is done by @see "devNew".
 #
-#  Also, for certain devices (eps, pdf, postscript, quartz, windows and x11), 
+#  Also, for certain devices (eps, pdf, postscript, quartz, windows and x11),
 #  builtin R device options are set.
 # }
 #
@@ -52,8 +57,8 @@
 #
 # @keyword device
 # @keyword utilities
-#*/########################################################################### 
-devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jpeg2", "pdf", "pictex", "png", "png2", "postscript", "quartz", "svg", "tiff", "windows", "x11", "xfig"), custom=TRUE, special=TRUE, ..., reset=FALSE) {
+#*/###########################################################################
+devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jpeg2", "pdf", "pictex", "png", "png2", "postscript", "quartz", "svg", "tiff", "win.metafile", "windows", "x11", "xfig"), custom=TRUE, special=TRUE, drop=TRUE, options=list(), ..., reset=FALSE) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Local setups
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -72,6 +77,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
     quartz=c("grDevices::quartz"),
     svg=c("grDevices::svg"),
     tiff=c("grDevices::tiff"),
+    win.metafile=c("grDevices::win.metafile"),
     windows=c("grDevices::windows"),
     x11=c("grDevices::x11"),
     xfig=c("grDevices::xfig")
@@ -83,7 +89,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (.Platform$OS.type == "windows") {
     # To please R CMD check
-    windows.options <- NULL; rm("windows.options");
+    windows.options <- NULL; rm(list="windows.options");
     x11.options <- windows.options;
   }
 
@@ -158,7 +164,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
       letter    = c( 8.5 , 11   ),
       USr       = c(11   , 8.5  )
     );
-  
+
     paper <- tolower(options$paper);
     if (paper == "default") {
       paper <- getOption("papersize", "a4");
@@ -188,20 +194,27 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
   } # getDevOptions()
 
 
-  setDevOptions <- function(type, ..., reset=FALSE) {
+  setDevOptions <- function(.type, ..., reset=FALSE) {
     devOpts <- getOption("devOptions", list());
-    oopts <- opts <- devOpts[[type]];
+##    str(list(.type=.type, devOpts=devOpts));
+    oopts <- opts <- devOpts[[.type]];
+    if (is.null(opts)) opts <- list();
+    # Sanity check
+    stopifnot(is.list(opts));
+##    str(list(opts=opts));
     if (reset) {
       opts <- NULL;
     } else {
       args <- list(...);
       if (length(args) > 0L) {
         for (key in names(args)) {
-          opts[[key]] <- args[[key]];
+          value <- args[[key]];
+##          str(list(key=key, value=value))
+          opts[[key]] <- value;
         }
       }
     }
-    devOpts[[type]] <- opts;
+    devOpts[[.type]] <- opts;
     options(devOptions=devOpts);
     invisible(oopts);
   } # setDevOptions()
@@ -250,7 +263,62 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Argument 'options':
+  if (!is.list(options)) {
+    throw("Argument 'options' must be a list: ", class(options)[1L]);
+  }
+  nopts <- length(options);
+  if (nopts > 0L && is.null(names(options))) {
+    throw("Argument 'options' must be a named list.");
+  }
+
+  # Additional arguments
+  args <- list(...);
+  nargs <- length(args);
+  if (nargs > 0L && is.null(names(args))) {
+    throw("Optional ('...') arguments must be named.");
+  }
+
+  for (key in names(args)) {
+    options[[key]] <- args[[key]];
+  }
+  nopts <- length(options);
+  # Not needed anymore
+  args <- nargs <- NULL;
+
   # Argument 'type':
+  if (missing(type) || length(type) == 0L) {
+    knownTypes <- eval(formals(devOptions)$type);
+    if (nopts > 0L) {
+      throw("Cannot set device options. Argument 'type' is missing or NULL. Should be one of: ", paste(sprintf("'%s'", knownTypes), collapse=", "));
+    }
+
+    res <- devOptions(type=knownTypes, custom=custom, special=special, drop=drop, reset=reset);
+    if (reset) {
+      return(invisible(res));
+    } else {
+      return(res);
+    }
+  }
+
+  if (length(type) > 1L) {
+    if (nopts > 0L) {
+      throw("Cannot set device options for more than one devices at the time: ", hpaste(sprintf("'%s'", type), collapse=", "));
+    }
+
+    # Support vector of 'type':s
+    types <- type;
+    res <- sapply(types, FUN=devOptions, drop=TRUE);
+    fields <- lapply(res, FUN=function(opts) names(opts));
+    fields <- unique(unlist(fields, use.names=FALSE));
+    opts <- lapply(res, FUN=function(x) x[fields]);
+    opts <- unlist(opts, use.names=FALSE, recursive=FALSE);
+    dim(opts) <- c(length(fields), length(types));
+    dimnames(opts) <- list(fields, types);
+    opts <- t(opts);
+    ##class(opts) <- c("DeviceOptions", class(opts));
+    return(opts);
+  }
   if (is.function(type)) {
     # Try to find name of device function
     type <- findDeviceFunction(fcn=type);
@@ -267,15 +335,6 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
     throw("Cannot query/modify device options. Unknown device: ", type);
   }
 
-  # Additional arguments
-  args <- list(...);
-  nargs <- length(args);
-  if (nargs > 0L) {
-    if (is.null(names(args))) {
-      throw("Optional ('...') arguments must be named.");
-    }
-  }
-
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Locate the nnn.options() function for this type of device
@@ -294,7 +353,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
   # Reset?
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (reset) {
-    res <- devOptions(type=type, special=special, reset=FALSE);
+    res <- devOptions(type=type, special=special, drop=drop, reset=FALSE);
     setDevOptions(type, reset=TRUE);
 
     # Only for certain devices...
@@ -307,11 +366,11 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Assign user arguments, iff possible
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  if (nargs > 0L) {
-    do.call("setDevOptions", args=c(list(type), args));
-  
+  if (nopts > 0L) {
+    do.call("setDevOptions", args=c(list(.type=type), options));
+
     # Only for certain devices...
-    do.call("nnn.options", args=args);
+    do.call("nnn.options", args=options);
   }
 
 
@@ -389,9 +448,16 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
     }
   }
 
+  # Return a list?
+  if (!drop) {
+    names <- names(opts);
+    dim(opts) <- c(1L, length(opts));
+    dimnames(opts) <- list(type, names);
+    ## Possibly, class(opts) <- c("DeviceOptions", class(opts))
+  }
 
   # Return invisibly?
-  if (nargs > 0L) {
+  if (nopts > 0L) {
     invisible(opts);
   } else {
     opts;
@@ -400,7 +466,16 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "eps", "jpeg", "jp
 
 
 ############################################################################
-# HISTORY: 
+# HISTORY:
+# 2012-07-26
+# o Added arguments 'options' and 'drop' to devOptions().
+# 2012-07-24
+# o Now devOptions() supports a vector of devices types.  If so, then
+#   a matrix where each row specifies a device type.  The union of
+#   all possible fields are returned as columns.  Non-existing fields
+#   are returned as NULL.  If no device type is specified (or NULL), then
+#   all are returned.
+# o Now devOptions() also supports "win.metafile".
 # 2012-04-30
 # o Now devOptions() returns options invisibly if some options were set,
 #   otherwise not, e.g. devOptions() versus devOptions("png", width=1024).
