@@ -1,18 +1,22 @@
 ###########################################################################
 ## This 'tangle' R script was created from an RSP document.
 ## RSP source document: 'R.devices-overview.tex.rsp'
+## Metadata 'title': 'R.devices overview'
+## Metadata 'author': 'Henrik Bengtsson'
+## Metadata 'keywords': 'devices, graphics, plots, figures'
 ###########################################################################
 
+## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 t0 <- Sys.time()
 library("R.devices");
-pkg <- Package("R.devices");
 
 devOptions("png", width=840);
 options(width=85);
 options(digits=3);
 options(str=strOptions(strict.width="cut"));
-getName(pkg)
-getAuthor(pkg)
+R.rsp$version
+R.rsp$author
+format(as.Date(getDate(R.devices)), format="%B %d, %Y")
 envir <- getNamespace("R.devices");
 tos <- ls(pattern="^to[A-Z][a-zA-Z]+", envir=envir);
 tos <- sprintf("%s()", tos);
@@ -31,12 +35,7 @@ evalWithEcho({
 devOptions("png", reset=TRUE)
 })
 evalWithEcho({
-knownDevices <- eval(formals(devOptions)$type)
-fields <- c("width", "height", "bg", "fg", "pointsize")
-opts <- sapply(knownDevices, FUN=function(type) devOptions(type)[fields])
-rownames(opts) <- fields
-opts <- t(opts)
-print(opts)
+devOptions()[,c("width", "height", "bg", "fg", "pointsize")]
 })
 toLatex(sessionInfo())
 dt <- round(Sys.time()-t0, digits=2)
